@@ -252,9 +252,7 @@ class PromptManager:
                 return f'#{title}#'+'\n\n'+f'{data}'
             return ''
         def get_chunk_header(current_chunk,total_chunks,chunk_data):
-    
-            if total_chunks in [0,'0'] or eatAll(chunk_data,['\n','\t',' ','']) in['',None]:
-                
+            if total_chunks in [0,'0'] or chunk_data in ['',None]:
                 return None
             current_chunk = current_chunk or 0
             try:
@@ -285,8 +283,6 @@ class PromptManager:
             data = value['data']
             if data:
                 self.key_check.append(key)
-                if key == 'chunk_data':
-                    self.last_chunk = value['data']
                 prompt_data_source+=f"{get_delimeters()}{get_for_prompt(title,data)}"
         return prompt_data_source
 
@@ -332,8 +328,8 @@ class PromptManager:
                                               prompt_as_previous=prompt_as_previous)
         max_tokens = chunk_token_distribution_data['completion']['available']
         self.prompt ={"model": self.model_mgr.selected_model_name, "messages": [{"role": self.role or "user", "content":self.prompt }],"max_tokens": max_tokens}
-        for each in self.key_check:
-            if each not in ['instructions']:
+        for key in self.key_check:
+            if key not in ['instructions']:
                 break
             self.prompt['blank_prompt']=True
         return self.prompt
