@@ -1,46 +1,174 @@
 # DEPENDENCIES #
 import os
 import pyperclip
-from .. import (ApiManager,
-                ModelManager,
-                PromptManager,
-                ResponseManager,
-                SaveManager,
-                InstructionManager,
-                read_from_file_with_multiple_encodings,
-                FileCollator)
+from .. import (
+    ApiManager,
+    ModelManager,
+    PromptManager,
+    ResponseManager,
+    SaveManager,
+    InstructionManager,
+    read_from_file_with_multiple_encodings,
+    FileCollator
+)
 
-from .abstract_ai_gui import (get_total_layout,
-                              instructions_keys,
-                              all_token_keys,
-                              test_options_keys)
-
-from .abstract_gui_navigation import AbstractNavigationManager
+from .abstract_ai_gui import get_total_layout,instructions_keys, all_token_keys, test_options_keys
 
 from .generate_readme import read_me_window
 
-from abstract_webtools import (UrlManager,
-                               SafeRequest,
-                               url_grabber_component)
+from abstract_webtools import UrlManager, SafeRequest, url_grabber_component
 
-from abstract_gui import (get_event_key_js,
-                          text_to_key,
-                          AbstractWindowManager,
-                          NextReadManager,
-                          AbstractBrowser)
+from abstract_gui import (
+    get_event_key_js,
+    text_to_key,
+    AbstractWindowManager,
+    NextReadManager,
+    AbstractBrowser
+)
+from abstract_utilities import (
+    get_any_value,
+    create_new_name,
+    get_sleep,
+    eatAll,
+    safe_json_loads,
+    read_from_file,
+    make_list,
+    ThreadManager,
+    HistoryManager,
+    get_file_create_time,
+    safe_read_from_json,
+    is_number
+)
 
-from abstract_utilities import (get_any_value,
-                                create_new_name,
-                                get_sleep,
-                                eatAll,
-                                safe_json_loads,
-                                read_from_file,
-                                make_list,
-                                ThreadManager,
-                                HistoryManager,
-                                get_file_create_time,
-                                safe_read_from_json,
-                                is_number)
+
+
+class AbstractNavigationManager:  # Replace with your actual class name
+    def __init__(self,selfs,window_mgr):
+        self.window_mgr = window_mgr
+        self.selfs =selfs
+    
+    def chunk_event_check(self):
+        """
+        Checks if any events related to the chunk UI elements were triggered and performs the necessary actions.
+        """
+        self.event,self.values,self.window=args
+        # Simplified method to determine the type and direction of navigation
+        navigation_type, navigation_direction = self.parse_navigation_event(self.event)
+
+        # Early exit if the event is not a navigation event
+        if not navigation_type:
+            return
+
+        # Retrieve navigation data based on the event
+        nav_data = self.get_navigation_data(navigation_type)
+
+        # Update the section and subsection numbers based on navigation
+        self.update_navigation_counters(nav_data, navigation_direction)
+
+        # Update the display based on the updated navigation data
+        self.update_display(nav_data)
+
+    def parse_navigation_event(self, event):
+        """
+        Parses the event to extract navigation type and direction.
+        """
+        if event.startswith('-') and event.endswith('-'):
+            parts = event[1:-1].lower().split('_')
+            if 'back' in parts or 'forward' in parts:
+                nav_type = '_'.join(parts[:-2]) if 'section' in parts else parts[0]
+                nav_direction = parts[-1]
+                return nav_type, nav_direction
+        return None, None
+
+    def get_navigation_data(self, navigation_type):
+        """
+        Retrieves the necessary data for navigation based on the navigation type.
+        """
+        nav_data = {
+            'data_type': navigation_type,
+            'section_number': int(self.window_mgr.get_from_value(f"-{navigation_type.upper()}_SECTION_NUMBER-")),
+            'number': self.get_sub_section_number(navigation_type),
+            'reference_object': self.get_reference_object(navigation_type)
+        }
+        return nav_data
+
+    def get_sub_section_number(self, navigation_type):
+        """
+        Retrieves the current sub-section number based on navigation type.
+        """
+        spl = self.window_mgr.get_event().split('_')
+        section = spl[1]
+        self.number_key = f"-{navigation_type.upper()}{'_SECTION' if len(spl)>= 3 else ''}_NUMBER-"
+        return int(self.window_mgr.get_from_value(self.number_key)) if self.window_mgr.exists(self.number_key) else None
+
+    def get_reference_object(self, navigation_type):
+        """
+        Retrieves the reference object based on navigation type.
+        """
+        reference_js = {
+            "request": self.selfs.request_data_list,
+            "prompt_data": self.selfs.prompt_data_list,
+            "chunk": self.selfs.prompt_mgr.chunk_token_distributions,
+            "query": self.selfs.prompt_mgr.chunk_token_distributions
+        }
+        return reference_js.get(navigation_type, [])
+
+    def update_navigation_counters(self, nav_data, direction):
+        """
+        Updates section and subsection numbers based on the navigation direction.
+        """
+        def get_adjusted_number(current_number, reference_obj):
+            return max(0, min(current_number, max(0, len(reference_obj))))
+        reference_obj = nav_data['reference_object']
+        current_section_number = self.selfs.display_number_tracker['prompt_data']
+        current_section_number = get_adjusted_number(current_section_number, reference_obj)
+        if 'SECTION' in self.number_key:
+            max_value = max(0, len(reference_obj) - 1)
+            increment = 1 if direction == 'forward' else -1
+            nav_data['section_number'] = max(0, min(current_section_number + increment, max_value))
+            nav_data["number"]=0
+        elif nav_data['data_type'] in ["chunk", "query"]:
+            reference_obj = reference_obj[current_section_number]
+            current_chunk_number = self.selfs.display_number_tracker['chunk_number']
+            current_chunk_number = get_adjusted_number(current_chunk_number, reference_obj)
+
+            max_value = max(0, len(reference_obj) - 1)
+            increment = 1 if direction == 'forward' else -1
+            nav_data['number'] = max(0, min(current_chunk_number + increment, max_value))
+
+        self.update_display(nav_data)
+        
+    def calculate_new_counter(self, current_counter, direction,min_value, max_value):
+        """
+        Calculates the new counter value based on the current value, direction, and maximum allowed value.
+        """
+        if direction == 'forward':
+            return min(current_counter + 1, max_value)
+        elif direction == 'back':
+            return max(current_counter - 1, min_value)
+        return current_counter
+
+    def update_display(self, nav_data):
+        """
+        Updates the display based on the navigation data.
+        """
+        self.window_mgr.update_value(key=text_to_key("prompt_data section number"),value=nav_data["section_number"])
+        self.window_mgr.update_value(key=text_to_key("request section number"),value=nav_data["section_number"])
+        self.selfs.update_request_data_display(nav_data["section_number"])
+        self.selfs.update_prompt_data_display(nav_data["section_number"])
+        self.selfs.display_number_tracker['prompt_data']=nav_data["section_number"]
+        self.selfs.display_number_tracker['request']=nav_data["section_number"]
+        self.window_mgr.update_value('-CHUNK_SECTION_NUMBER-',nav_data['section_number'])
+        self.window_mgr.update_value('-QUERY_SECTION_NUMBER-',nav_data['section_number'])
+        self.window_mgr.update_value("-CHUNK_SECTIONED_DATA-",self.selfs.prompt_mgr.chunk_token_distributions[nav_data["section_number"]][0]['chunk']['data'])
+        self.window_mgr.update_value(key='-QUERY-',value=self.selfs.prompt_mgr.create_prompt(chunk_token_distribution_number=nav_data['section_number'],chunk_number=0))
+        self.selfs.display_number_tracker['query']=nav_data["section_number"]
+        self.selfs.display_number_tracker['chunk']=nav_data["section_number"]
+        self.window_mgr.update_value('-QUERY_NUMBER-',nav_data["number"])
+        self.window_mgr.update_value('-CHUNK_NUMBER-',nav_data["number"])
+        self.selfs.display_number_tracker['chunk_number']=nav_data["number"]
+        self.selfs.update_chunk_info(nav_data['section_number'],nav_data["number"])
+        # Update the display logic here based on nav_data
 
 class GptManager:
     """
@@ -130,30 +258,17 @@ class GptManager:
         AI's responses, the instruction handling, and the submission of queries to the AI. It finally sets a few more internal state variables
         and kicks off the UI's main loop.
  	"""
-        
+        self.window_mgr = AbstractWindowManager()
+        self.window_name = self.window_mgr.add_window(window_name="Chat GPT Console",title="Chat GPT Console",layout=get_total_layout())
+        self.window_mgr.set_current_window(self.window_name)
+        self.window = self.window_mgr.get_window_method(self.window_name)
+        self.api_call_list=[]
+        self.instruction_bool_keys=[]
         self.values = None
         self.event = None
-        
         self.chunk_title=None
         self.chunk_number=None
-        
-        self.output_list=[]
-        self.latest_output=[]
-        self.prompt_data_list=['']
-        self.request_data_list=['']
-        self.new_response_path_list=[]
-        self.response_directories_list=[]
-        
-        #self.response_path_list = self.aggregate_conversations()
-        
-        self.response=False
-        self.test_bool=False
-        self.loop_one=False
-        self.bool_loop=False
         self.start_query=False
-        self.updated_progress = False
-        self.api_call_list=[]
-        
         self.display_number_tracker={'instructions':0,'request':0,'prompt_data':0,'chunk':0,'query':0,'chunk_number':0}
         self.navigation_mgr= AbstractNavigationManager(self,window_mgr=self.window_mgr,)
         self.browser_mgr = AbstractBrowser(window_mgr=self)
@@ -162,25 +277,23 @@ class GptManager:
         self.history_mgr = HistoryManager()
         self.model_mgr = ModelManager()
         self.instruction_mgr = InstructionManager()
-
-        self.initialize_keys()
         
-        self.window_mgr = AbstractWindowManager()
-        self.window_name = self.window_mgr.add_window(window_name="Chat GPT Console",title="Chat GPT Console",layout=get_total_layout())
-        self.window_mgr.set_current_window(self.window_name)
-        self.window = self.window_mgr.get_window_method(self.window_name)
+        self.response=False
+        self.updated_progress = False
+        self.test_bool=False
+        self.output_list=[]
+        self.latest_output=[]
+        self.request_data_list=['']
+        self.prompt_data_list=['']
+        self.instructions_list=[{}]
+        self.new_response_path_list=[]
+        self.response_directories_list=[]
+        self.response_path_list = self.aggregate_conversations()
+        self.initialize_keys()
+        self.loop_one=False
+        self.bool_loop=False
         self.window_mgr.while_window(window_name=self.window_name, event_handlers=self.while_window)
-
-    def unlist_self(self,keys):
-        """
-        A helper method that takes a list of keys as a parameter, iterates over each key, and checks if
-        the value of the corresponding key in the class is a non-empty list. If it is, it updates the class's attribute with the last element in the list.
- 	"""
-        for key in keys:
-            value = getattr(self,key)
-            if isinstance(value,list) and value:
-                value = value[-1]
-            setattr(self,key,value)
+        
     @staticmethod
     def get_remainder(num):
         return 100-int(num)
@@ -221,8 +334,6 @@ class GptManager:
         needed to check the event, the chunk history, the token percentage dropdowns, the instruction keys, and others.
         This is essential for the proper functioning and interaction of the GptManager with the GUI.
  	"""
-        self.instruction_bool_keys=[]
-        
         self.event_check_keys = ['chunk_event_check',
                                  'response_event_check',
                                  'url_event_check',
@@ -234,42 +345,20 @@ class GptManager:
                                  'readme_event_check',
                                  'browser_event_query',
                                  'prompt_request_event_check']
-        
-        self.sub_navigation_keys = [text_to_key("chunk back"),
-                                    text_to_key("chunk forward"),
-                                    text_to_key('chunk section back'),
-                                    text_to_key('chunk section forward'),
-                                    text_to_key("query back"),
-                                    text_to_key("query forward"),
-                                    text_to_key('query section back'),
-                                    text_to_key('query section forward'),]
-        
-        self.section_navigation_keys = [text_to_key('instructions section back'),
-                                        text_to_key('instructions section forward'),
-                                        text_to_key('request section back'),
-                                        text_to_key('request section forward'),
-                                        text_to_key('prompt_data section back'),
-                                        text_to_key('prompt_data section forward'),]
-        
-        self.section_navigation_keys = self.section_navigation_keys+self.sub_navigation_keys
-        
         self.chunk_history_name = self.history_mgr.add_history_name('chunk')
-        
         self.toke_percentage_dropdowns = ['-COMPLETION_PERCENTAGE-',
                                           '-PROMPT_PERCENTAGE-']
-        
         self.additions_key_list = self.browser_mgr.key_list+['-FILE_TEXT-',
                                                              '-ADD_FILE_TO_CHUNK-',
                                                              '-ADD_URL_TO_CHUNK-']
-        
         self.instruction_pre_keys = instructions_keys
         for key in self.instruction_pre_keys:
             self.instruction_bool_keys.append(text_to_key(text=key,section='bool'))
-            
         self.sectioned_chunk_text_number_key= text_to_key('chunk text number')
         self.sectioned_chunk_data_key = text_to_key('chunk sectioned data')
-        self.chunk_display_keys=self.get_bool_and_text_keys(all_token_keys)
-        
+        self.chunk_display_keys=self.get_bool_and_text_keys(
+            all_token_keys
+            )
     def update_all(self):
         """
         updates all the needed managers. It updates the Model Manager, Instruction Manager, API Manager,
@@ -290,6 +379,7 @@ class GptManager:
         if self.prompt_data_list[-1] != '' or self.request_data_list[-1] != '':
             self.prompt_data_list.append('')
             self.request_data_list.append('')
+            self.instructions_list.append({})
             self.update_text()
             
 #ApiManagement:
@@ -332,12 +422,6 @@ class GptManager:
         """
         initializes the prompt manager with the information needed for querying the GPT AI model.
 	"""
-        for i in range(len(self.request_data_list)):
-            if self.prompt_data_list[-1] == '' and self.request_data_list[-1] == '':
-                self.prompt_data_list[:-1]
-                self.request_data_list[:-1]
-            else:
-                break
         self.prompt_mgr = PromptManager(instruction_mgr=self.instruction_mgr,
                                    model_mgr=self.model_mgr,
                                    role=self.window_mgr.get_from_value('-ROLE-'),
@@ -351,19 +435,24 @@ class GptManager:
         chunk_token_distribution_number = self.get_spec_section_number_display('query')
         chunk_number = self.display_number_tracker['chunk_number']
         distributions = self.prompt_mgr.chunk_token_distributions
-        if distributions:
-            if len(distributions)-1<=chunk_token_distribution_number:
-                chunk_token_distribution_number =0
-            chunk_distribution = distributions[chunk_token_distribution_number]
-            if chunk_distribution:
-                if len(chunk_distribution)-1 <= chunk_number:
-                    chunk_number = 0
-            self.chunk_token_distributions=self.prompt_mgr.chunk_token_distributions
-            self.update_chunk_info(chunk_token_distribution_number,chunk_number)
+        if len(distributions)-1<=chunk_token_distribution_number:
+            chunk_token_distribution_number =0
+        chunk_distribution = distributions[chunk_token_distribution_number]
+        if chunk_distribution:
+            if len(chunk_distribution)-1 <= chunk_number:
+                chunk_number = 0
+        self.chunk_token_distributions=self.prompt_mgr.chunk_token_distributions
+        self.update_chunk_info(chunk_token_distribution_number,chunk_number)
         
     def get_prompt_data_section_number(self):
         return int(self.window_mgr.get_values()[text_to_key("prompt_data section number")])
     
+    def get_chunk_token_distribution_number(self):
+        return int(self.window_mgr.get_values()[text_to_key("chunk section number")])
+    
+    def get_chunk_number(self):
+        return int(self.window_mgr.get_values()[text_to_key("chunk number")])
+    
     def get_spec_section_number_display(self,variable_header):
         value = self.display_number_tracker[variable_header]
         if not is_number(value):
@@ -476,8 +565,25 @@ class GptManager:
         """
         checks if any events related to the chunk UI elements were triggered and performs the necessary actions.
  	"""
-        
-        if self.event in self.section_navigation_keys:
+        sub_navigation_keys = [
+                          text_to_key("chunk forward"),
+                          text_to_key("chunk back"),
+                          text_to_key("query forward"),
+                          text_to_key("query back"),
+                          text_to_key('chunk section forward'),
+                          text_to_key('query section forward'),
+                          text_to_key('chunk section back'),
+                          text_to_key('query section back'),
+                          ]
+        section_navigation_keys = [
+                          text_to_key('instructions section back'),
+                          text_to_key('instructions section forward'),
+                          text_to_key('request section back'),
+                          text_to_key('request section forward'),
+                          text_to_key('prompt_data section back'),
+                          text_to_key('prompt_data section forward'),
+                          ]
+        if self.event in section_navigation_keys+sub_navigation_keys:
             nav_type, nav_direction=self.navigation_mgr.parse_navigation_event(self.event)
             nav_data=self.navigation_mgr.get_navigation_data(nav_type)
             sub_section_number = self.navigation_mgr.get_sub_section_number(nav_type)
@@ -592,6 +698,7 @@ class GptManager:
  	"""
         if self.event in self.instruction_bool_keys:
             self.update_text()
+
         else:
             return False
         return True
@@ -877,7 +984,16 @@ class GptManager:
                     return len(request_data_list)-1
                 return i
         return len(self.request_data_list)-1
-
+    def unlist_self(self,keys):
+        """
+        A helper method that takes a list of keys as a parameter, iterates over each key, and checks if
+        the value of the corresponding key in the class is a non-empty list. If it is, it updates the class's attribute with the last element in the list.
+ 	"""
+        for key in keys:
+            value = getattr(self,key)
+            if isinstance(value,list) and value:
+                value = value[-1]
+            setattr(self,key,value)
     def initialize_output_display(self):
         """
         Initializes the display of the latest output in the GUI.
@@ -934,40 +1050,39 @@ class GptManager:
         Returns:
             bool: True if a listbox event has been triggered, otherwise False
         """
-        def open_folder():
-            file_path = self.window_mgr.get_values()[self.script_event_js['-DIR-']]
-            files_list = self.window_mgr.get_values()[self.script_event_js['-BROWSER_LIST-']]
-            if file_path:
-                file_path = os.path.join(file_path,files_list[0])
-                if not os.path.isfile(file_path):
-                    self.browser_mgr.handle_event(self.window_mgr.get_values(),self.event,self.window)
-        def open_file(file_path):
-            try:
-                contents = self.read_file_contents(file_path)  # Assuming read_file_contents is defined to handle different file types
-                self.window_mgr.update_value(key=self.script_event_js['-FILE_TEXT-'], value=safe_json_loads(contents))
-                self.chunk_title = os.path.basename(file_path)
-                self.window_mgr.update_value(key=text_to_key('-CHUNK_TITLE-', section='files'), value=self.chunk_title)
-            except Exception as e:
-                print(f"Cannot read from path {file_path}. Error: {e}")
         event_triggered = False
+        values = self.window_mgr.get_values()
         file_path = None
+
         if self.script_event_js['found'] == '-FILES_LIST-':
-            selected_files = self.values[self.script_event_js['-FILES_LIST-']]
+            selected_files = values[self.event]
             if selected_files:
                 file_path = selected_files[0]
                 event_triggered = True
         elif self.script_event_js['found'] == '-BROWSER_LIST-':
-            directory = self.values[self.script_event_js['-DIR-']]
-            selected_files = self.values[self.script_event_js['-BROWSER_LIST-']]
+            directory = values[self.script_event_js['-DIR-']]
+            selected_files = values[self.script_event_js['-BROWSER_LIST-']]
             if selected_files:
                 file_path = os.path.join(directory, selected_files[0])
                 event_triggered = True
+        
         if file_path:
             if os.path.isfile(file_path):
-                open_file(file_path)
+                try:
+                    contents = self.read_file_contents(file_path)  # Assuming read_file_contents is defined to handle different file types
+                    self.window_mgr.update_value(key=self.script_event_js['-FILE_TEXT-'], value=safe_json_loads(contents))
+                    self.chunk_title = os.path.basename(file_path)
+                    self.window_mgr.update_value(key=text_to_key('-CHUNK_TITLE-', section='files'), value=self.chunk_title)
+                except Exception as e:
+                    print(f"Cannot read from path {file_path}. Error: {e}")
             else:
-                open_folder()
-        return event_triggered
+                file_path = self.window_mgr.get_values()[self.script_event_js['-DIR-']]
+                files_list = self.window_mgr.get_values()[self.script_event_js['-BROWSER_LIST-']]
+                if file_path:
+                    file_path = os.path.join(file_path,files_list[0])
+                    if not os.path.isfile(file_path):
+                        self.browser_mgr.handle_event(self.window_mgr.get_values(),self.event,self.window)
+            return event_triggered
     
     def update_last_response_file(self):
         """
@@ -1109,13 +1224,10 @@ class GptManager:
             for browser_event in ['-SCAN_FILES-',
                                   '-SCAN_RESPONSES-']:
                 self.browser_event_query(self.values,browser_event,self.window)
-        try:
-            self.next_read_mgr.execute_queue()
-            self.script_event_js = get_event_key_js(event = self.event,key_list=self.additions_key_list)
-            self.perform_event_checks()
-            self.loop_one=True
-        except:
-            pass
+        self.next_read_mgr.execute_queue()
+        self.script_event_js = get_event_key_js(event = self.event,key_list=self.additions_key_list)
+        self.perform_event_checks()
+        
 #TestManagement
     def check_test_bool(self):
         """
